/* 
 * OpenTyrian Classic: A modern cross-platform port of Tyrian
 * Copyright (C) 2007-2009  The OpenTyrian Development Team
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include "joystick.h"
#include "keyboard.h"
#include "network.h"
#include "opentyr.h"
#include "video.h"
#include "video_scale.h"

#include "SDL.h"

#ifdef GP2X_WIZ

#include "nortsong.h"

enum WizButtons {
		WIZ_BUTTON_UP, WIZ_BUTTON_UPLEFT, WIZ_BUTTON_LEFT,
		WIZ_BUTTON_DOWNLEFT, WIZ_BUTTON_DOWN,
		WIZ_BUTTON_DOWNRIGHT, WIZ_BUTTON_RIGHT,
		WIZ_BUTTON_UPRIGHT, WIZ_BUTTON_START, WIZ_BUTTON_SELECT,
		WIZ_BUTTON_L, WIZ_BUTTON_R, WIZ_BUTTON_A, WIZ_BUTTON_B,
		WIZ_BUTTON_X, WIZ_BUTTON_Y, WIZ_BUTTON_VOLUP,
		WIZ_BUTTON_VOLDOWN
	};
Uint8 wiz_dir_pressed[8] = {false, false, false, false, 
							false, false, false, false};
#endif

JE_boolean ESCPressed;

JE_boolean newkey, newmouse, keydown, mousedown;
SDLKey lastkey_sym;
SDLMod lastkey_mod;
unsigned char lastkey_char;
Uint8 lastmouse_but;
Uint16 lastmouse_x, lastmouse_y;
JE_boolean mouse_pressed[3] = {false, false, false};
Uint16 mouse_x, mouse_y, mouse_xrel, mouse_yrel;

int numkeys;
#ifndef GP2X_WIZ
Uint8 *keysactive;
#else
Uint8 keysactive[SDLK_LAST];
#endif

#ifdef NDEBUG
bool input_grab_enabled = true,
#else
bool input_grab_enabled = false,
#endif
     input_grabbed = false;


void flush_events_buffer( void )
{
	SDL_Event ev;

	while (SDL_PollEvent(&ev));
}

void wait_input( JE_boolean keyboard, JE_boolean mouse, JE_boolean joystick )
{
	service_SDL_events(false);
	while (!((keyboard && keydown) || (mouse && mousedown) || (joystick && joydown)))
	{
		SDL_Delay(SDL_POLL_INTERVAL);
		push_joysticks_as_keyboard();
		service_SDL_events(false);
		
		if (isNetworkGame)
			network_check();
	}
}

void wait_noinput( JE_boolean keyboard, JE_boolean mouse, JE_boolean joystick )
{
	service_SDL_events(false);
	while ((keyboard && keydown) || (mouse && mousedown) || (joystick && joydown))
	{
		SDL_Delay(SDL_POLL_INTERVAL);
		poll_joysticks();
		service_SDL_events(false);
		
		if (isNetworkGame)
			network_check();
	}
}

void init_keyboard( void )
{
#ifndef GP2X_WIZ
	keysactive = SDL_GetKeyState(&numkeys);
#else
	memset(keysactive, 0, sizeof(keysactive));
#endif
	SDL_EnableKeyRepeat(500, 60);

	newkey = newmouse = false;
	keydown = mousedown = false;

	SDL_EnableUNICODE(1);
}

void input_grab( void )
{
#ifdef TARGET_GP2X
	input_grabbed = true;
#else /* TARGET_GP2X */
	input_grabbed = input_grab_enabled || fullscreen_enabled;
#endif /* TARGET_GP2X */
	
	SDL_ShowCursor(input_grabbed ? SDL_DISABLE : SDL_ENABLE);
#ifdef NDEBUG
	SDL_WM_GrabInput(input_grabbed ? SDL_GRAB_ON : SDL_GRAB_OFF);
#endif
}

JE_word JE_mousePosition( JE_word *mouseX, JE_word *mouseY )
{
	service_SDL_events(false);
	*mouseX = mouse_x;
	*mouseY = mouse_y;
	return mousedown ? lastmouse_but : 0;
}

void set_mouse_position( int x, int y )
{
#ifndef GP2X_WIZ
	if (input_grabbed)
		SDL_WarpMouse(x * scale, y * scale);
#endif
}

void service_SDL_events( JE_boolean clear_new )
{
	SDL_Event ev;
	
	if (clear_new)
		newkey = newmouse = false;
	
	while (SDL_PollEvent(&ev))
	{
		switch (ev.type)
		{
#ifndef GP2X_WIZ
			case SDL_MOUSEMOTION:
				mouse_x = ev.motion.x / scale;
				mouse_y = ev.motion.y / scale;
				mouse_xrel = ev.motion.xrel / scale;
				mouse_yrel = ev.motion.yrel / scale;
				break;
#endif
			case SDL_KEYDOWN:
				if (ev.key.keysym.mod & KMOD_CTRL)
				{
					/* <ctrl><bksp> emergency kill */
					if (ev.key.keysym.sym == SDLK_BACKSPACE)
					{
						puts("\n\n\nCtrl+Backspace pressed. Doing emergency quit.\n");
						SDL_Quit();
						exit(1);
					}
					
					/* <ctrl><f10> toggle input grab */
					if (ev.key.keysym.sym == SDLK_F10)
					{
						input_grab_enabled = !input_grab_enabled;
						input_grab();
						break;
					}
				}
				
				if (ev.key.keysym.mod & KMOD_ALT)
				{
					/* <alt><enter> toggle fullscreen */
					if (ev.key.keysym.sym == SDLK_RETURN)
					{
						fullscreen_enabled = !fullscreen_enabled;
						reinit_video();
						break;
					}
					
					/* <alt><tab> disable input grab and fullscreen */
					if (ev.key.keysym.sym == SDLK_TAB)
					{
						input_grab_enabled = false;
						input_grab();
						
						fullscreen_enabled = false;
						reinit_video();
						break;
					}
				}
				
				newkey = true;
				lastkey_sym = ev.key.keysym.sym;
				lastkey_mod = ev.key.keysym.mod;
				lastkey_char = ev.key.keysym.unicode;
				keydown = true;
				return;
			case SDL_KEYUP:
				keydown = false;
				return;
#ifndef GP2X_WIZ
			case SDL_MOUSEBUTTONDOWN:
				if (!input_grabbed)
				{
					input_grab_enabled = !input_grab_enabled;
					input_grab();
					break;
				}
			case SDL_MOUSEBUTTONUP:
				if (ev.type == SDL_MOUSEBUTTONDOWN)
				{
					newmouse = true;
					lastmouse_but = ev.button.button;
					lastmouse_x = ev.button.x / scale;
					lastmouse_y = ev.button.y / scale;
					mousedown = true;
				}
				else
				{
					mousedown = false;
				}
				switch (ev.button.button)
				{
					case SDL_BUTTON_LEFT:
						mouse_pressed[0] = mousedown; break;
					case SDL_BUTTON_RIGHT:
						mouse_pressed[1] = mousedown; break;
					case SDL_BUTTON_MIDDLE:
						mouse_pressed[2] = mousedown; break;
				}
				break;
#else
		case SDL_JOYBUTTONDOWN:
			newkey = true;
			keydown = true;
			switch (ev.jbutton.button)
			{
			case WIZ_BUTTON_UP:
				lastkey_sym = SDLK_UP;
				keysactive[SDLK_UP] = true;
				wiz_dir_pressed[WIZ_BUTTON_UP] = true;
				break;
			case WIZ_BUTTON_DOWN:
				lastkey_sym = SDLK_DOWN;
				keysactive[SDLK_DOWN] = true;
				wiz_dir_pressed[WIZ_BUTTON_DOWN] = true;
				break;
			case WIZ_BUTTON_LEFT:
				lastkey_sym = SDLK_LEFT;
				keysactive[SDLK_LEFT] = true;
				wiz_dir_pressed[WIZ_BUTTON_LEFT] = true;
				break;
			case WIZ_BUTTON_RIGHT:
				lastkey_sym = SDLK_RIGHT;
				keysactive[SDLK_RIGHT] = true;
				wiz_dir_pressed[WIZ_BUTTON_RIGHT] = true;
				break;
			case WIZ_BUTTON_UPLEFT:
				keysactive[SDLK_UP] = true;
				keysactive[SDLK_LEFT] = true;
				wiz_dir_pressed[WIZ_BUTTON_UPLEFT] = true;
				break;
			case WIZ_BUTTON_UPRIGHT:
				keysactive[SDLK_UP] = true;
				keysactive[SDLK_RIGHT] = true;
				wiz_dir_pressed[WIZ_BUTTON_UPRIGHT] = true;
				break;
			case WIZ_BUTTON_DOWNLEFT:
				keysactive[SDLK_DOWN] = true;
				keysactive[SDLK_LEFT] = true;
				wiz_dir_pressed[WIZ_BUTTON_DOWNLEFT] = true;
				break;
			case WIZ_BUTTON_DOWNRIGHT:
				keysactive[SDLK_DOWN] = true;
				keysactive[SDLK_RIGHT] = true;
				wiz_dir_pressed[WIZ_BUTTON_DOWNRIGHT] = true;
				break;
			case WIZ_BUTTON_SELECT:
				lastkey_sym = SDLK_RETURN;
				keysactive[SDLK_RETURN] = true;
				break;
			case WIZ_BUTTON_A:
				lastkey_sym = SDLK_RETURN;
				keysactive[SDLK_RETURN] = true;
				break;
			case WIZ_BUTTON_B:
				lastkey_sym = SDLK_SPACE;
				keysactive[SDLK_SPACE] = true;
				break;
			case WIZ_BUTTON_START:
			case WIZ_BUTTON_X:
				lastkey_sym = SDLK_ESCAPE;
				keysactive[SDLK_ESCAPE] = true;
				break;
			case WIZ_BUTTON_L:
				lastkey_sym = SDLK_LCTRL;
				keysactive[SDLK_LCTRL] = true;
				break;
			case WIZ_BUTTON_R:
				lastkey_sym = SDLK_LALT;
				keysactive[SDLK_LALT] = true;
				break;
			case WIZ_BUTTON_VOLUP:
				JE_changeVolume(&tyrMusicVolume, 12, &fxVolume, 12);
				lastkey_sym = SDLK_UNKNOWN;
				break;
			case WIZ_BUTTON_VOLDOWN:
				JE_changeVolume(&tyrMusicVolume, -12, &fxVolume, -12);
				lastkey_sym = SDLK_UNKNOWN;
				break;
			default:
				lastkey_sym = SDLK_UNKNOWN;
				break;
			}
			break;
		case SDL_JOYBUTTONUP:
			keydown = false;
			switch (ev.jbutton.button)
			{
			case WIZ_BUTTON_UP:
				keysactive[SDLK_UP] = (wiz_dir_pressed[WIZ_BUTTON_UPLEFT] || wiz_dir_pressed[WIZ_BUTTON_UPRIGHT]);
				wiz_dir_pressed[WIZ_BUTTON_UP] = false;
				break;
			case WIZ_BUTTON_DOWN:
				keysactive[SDLK_DOWN] = (wiz_dir_pressed[WIZ_BUTTON_DOWNLEFT] || wiz_dir_pressed[WIZ_BUTTON_DOWNRIGHT]);
				wiz_dir_pressed[WIZ_BUTTON_DOWN] = false;
				break;
			case WIZ_BUTTON_LEFT:
				keysactive[SDLK_LEFT] = (wiz_dir_pressed[WIZ_BUTTON_UPLEFT] || wiz_dir_pressed[WIZ_BUTTON_DOWNLEFT]);
				wiz_dir_pressed[WIZ_BUTTON_LEFT] = false;
				break;
			case WIZ_BUTTON_RIGHT:
				keysactive[SDLK_RIGHT] = (wiz_dir_pressed[WIZ_BUTTON_UPRIGHT] || wiz_dir_pressed[WIZ_BUTTON_DOWNRIGHT]);
				wiz_dir_pressed[WIZ_BUTTON_RIGHT] = false;
				break;
			case WIZ_BUTTON_UPLEFT:
				keysactive[SDLK_UP] = (wiz_dir_pressed[WIZ_BUTTON_UP] || wiz_dir_pressed[WIZ_BUTTON_UPRIGHT]);
				keysactive[SDLK_LEFT] = (wiz_dir_pressed[WIZ_BUTTON_LEFT] || wiz_dir_pressed[WIZ_BUTTON_DOWNLEFT]);
				wiz_dir_pressed[WIZ_BUTTON_UPLEFT] = false;
				break;
			case WIZ_BUTTON_UPRIGHT:
				keysactive[SDLK_UP] = (wiz_dir_pressed[WIZ_BUTTON_UP] || wiz_dir_pressed[WIZ_BUTTON_UPLEFT]);
				keysactive[SDLK_RIGHT] = (wiz_dir_pressed[WIZ_BUTTON_RIGHT] || wiz_dir_pressed[WIZ_BUTTON_DOWNRIGHT]);
				wiz_dir_pressed[WIZ_BUTTON_UPRIGHT] = false;
				break;
			case WIZ_BUTTON_DOWNLEFT:
				keysactive[SDLK_DOWN] = (wiz_dir_pressed[WIZ_BUTTON_DOWN] || wiz_dir_pressed[WIZ_BUTTON_DOWNRIGHT]);
				keysactive[SDLK_LEFT] = (wiz_dir_pressed[WIZ_BUTTON_LEFT] || wiz_dir_pressed[WIZ_BUTTON_UPLEFT]);
				wiz_dir_pressed[WIZ_BUTTON_DOWNLEFT] = false;
				break;
			case WIZ_BUTTON_DOWNRIGHT:
				keysactive[SDLK_DOWN] = (wiz_dir_pressed[WIZ_BUTTON_DOWN] || wiz_dir_pressed[WIZ_BUTTON_DOWNLEFT]);
				keysactive[SDLK_RIGHT] = (wiz_dir_pressed[WIZ_BUTTON_RIGHT] || wiz_dir_pressed[WIZ_BUTTON_UPRIGHT]);
				wiz_dir_pressed[WIZ_BUTTON_DOWNRIGHT] = false;
				break;
			case WIZ_BUTTON_A:
				keysactive[SDLK_RETURN] = false;
				break;
			case WIZ_BUTTON_B:
				keysactive[SDLK_SPACE] = false;
				break;
			case WIZ_BUTTON_START:
			case WIZ_BUTTON_X:
				keysactive[SDLK_ESCAPE] = false;
				break;
			case WIZ_BUTTON_L:
				keysactive[SDLK_LCTRL] = false;
				break;
			case WIZ_BUTTON_R:
				keysactive[SDLK_LALT] = false;
				break;
			}
			break;
#endif
			case SDL_QUIT:
				/* TODO: Call the cleanup code here. */
				exit(0);
				break;
		}
	}
}

void JE_clearKeyboard( void )
{
	// /!\ Doesn't seems important. I think. D:
}

// kate: tab-width 4; vim: set noet:
